import { CommandHandler } from '@nestjs/cqrs';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { DeleteAuthCommand } from '../../commands/delete-auth.command';

@CommandHandler(DeleteAuthCommand)
export class DeleteAuthUseCase {
  constructor(
    private readonly _authRepository: AuthRepository,
    private readonly _usernameRepository: UsernameRepository,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: DeleteAuthCommand) {
    try {
      await this._usernameRepository.deleteMany({
        uuid: command.deleteAuthRequest.uuid,
      });
      return await this._authRepository.delete({
        uuid: command.deleteAuthRequest.uuid,
      });
    } catch (error) {
      this._loggingMessager.publish(
        'auth.delete.critical',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
